
set ns [new Simulator]

#Reads the configuration of the Piconet
source ./config.tcl

set Sim(repl) [new Classifier/Replicator]

#For traces...
if {$Sim(Trace) == "on"} {
	Baseband set namTrace_ 1
	Trace set namBTTrace_ 1
	set Sim(namFP) [open $Sim(NamFile) w]
	set Sim(trFP)  [open out.tr w]
	$ns namtrace-all $Sim(namFP)
}

#Reads the functions provided by 'aide.tcl'
source ./aide.tcl


################## Builds the master #####################
create-master $NumResponses [expr $InqTimeout*3200] [lindex $Sim(Lxpos) 0] [lindex $Sim(Lypos) 0]


setup-FPQ



################## Builds the slaves #####################
for {set i 1} {$i < $Sim(NumDevices)} {incr i} {
    set ISO [lindex $InqScanOffset [expr $i-1]]
    create-slave $i [expr $ISO*3200] [lindex $Sim(Lxpos) $i] [lindex $Sim(Lypos) $i]
}

################## Builds the Applications ###############
for {set i 1} {$i < $Sim(NumDevices)} {incr i} {
    set Trans  [lindex $Sim(LTrans_master) [expr $i-1]]
    set Appli  [lindex $Sim(LAppli_master) [expr $i-1]]
    set IdleT  [lindex $Sim(FR_master) [expr $i-1]]

    config-app-master-slave $i $Trans $Appli $IdleT

    set Trans  [lindex $Sim(LTrans_slave) [expr $i-1]]
    set Appli  [lindex $Sim(LAppli_slave) [expr $i-1]]
    set IdleT  [lindex $Sim(FR_slave) [expr $i-1]]
    
    config-app-slave-master $i $Trans $Appli $IdleT  
}


################## Setup of Traces ##################
if {$Sim(Trace) == "on"} {
	config-trace
	for {set i 1} {$i < $Sim(NumDevices)} {incr i} {
		puts $Sim(namFP) "l -t * -s 0 -d $i -S UP -r 5000000 -D .0005 -c black -o [expr 105+$i*30]deg"
	}
}


################# Starts the applications ############
start_appli


################# Starts the master and the slaves ####### 
set bthost [$BTmaster set bthost_]
$ns at [lindex $StartTime 0] "$bthost start"

for {set j 1} {$j < $Sim(NumDevices)} {incr j} {
	set bthost [$BTslave($j) set bthost_]
	$ns at [lindex $StartTime $j] "$bthost start"
}


############### END ##################
$ns at $SimulationTime "finish"


############## Simulation ############
$ns run
