
/*###############################################################################
#################################################################################
  This is an implementation of Poisson traffic based on Expoo traffic generator,
  and with the following distribution of IP packet sizes:
    - 30%  ->   40 Bytes
    - 58%  -> 1500 Bytes
    - 12%  -> 300 to 600 Bytes

  The only one parameter consists in the average time beetween each packet.
#################################################################################
#################################################################################/








\*
 * Copyright (c) Xerox Corporation 1997. All rights reserved.
 *  
 * License is granted to copy, to use, and to make and to use derivative
 * works for research and evaluation purposes, provided that Xerox is
 * acknowledged in all documentation pertaining to any such copy or derivative
 * work. Xerox grants no other licenses expressed or implied. The Xerox trade
 * name should not be used in any advertising without its written permission.
 *  
 * XEROX CORPORATION MAKES NO REPRESENTATIONS CONCERNING EITHER THE
 * MERCHANTABILITY OF THIS SOFTWARE OR THE SUITABILITY OF THIS SOFTWARE
 * FOR ANY PARTICULAR PURPOSE.  The software is provided "as is" without
 * express or implied warranty of any kind.
 *  
 * These notices must be retained in any copies of any part of this software.
 */

#ifndef lint
static const char rcsid[] =
    "@(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/expoo.cc,v 1.12 2000/10/13 19:44:15 debo Exp $ (Xerox)";
#endif

#include <stdlib.h>
 
#include "random.h"
#include "trafgen.h"
#include "ranvar.h"
#include <time.h>


// implements a Poisson traffic source based on Expoo traffic generator.  parameterized by average idle time,


class EXPOO_IP_Traffic : public TrafficGenerator {
 public:
	EXPOO_IP_Traffic();
	virtual double next_interval(int&);
        virtual void timeout();
 protected:
	void init();
	double interpackets_;       // average length of idle time (sec) 
	double MAX;

	// new stuff using RandomVariable
	//ExponentialRandomVariable burstlen_;
	ExponentialRandomVariable Offtime_;
};


static class EXPIPTrafficClass : public TclClass {
 public:
	EXPIPTrafficClass() : TclClass("Application/Traffic/Expo_IP") {}
	TclObject* create(int, const char*const*) {
		return (new EXPOO_IP_Traffic());
	}
} class_expoo_ip_traffic;



EXPOO_IP_Traffic::EXPOO_IP_Traffic() :  Offtime_(0.0)
{
	bind_time("interpackets_", Offtime_.avgp() );
}

void EXPOO_IP_Traffic::init()
{
        MAX  = (double) (RAND_MAX) + 1;
	srand( (unsigned) time( NULL ) );
	if (agent_)
	  agent_->set_pkttype(PT_EXP);
}

double EXPOO_IP_Traffic::next_interval(int& size)
{

        // Computes the size of the packet to be sent
	double t = (double) ( rand() ) / MAX;
	if (t < 0.3) {
	  size_ =40;
	  size = 40;
	}

	else {
	  if (t < 0.42) {
	    size_ = 1500;
	    size = 1500; }
	  else {
	    int ta = (int)(  (double) rand() *300 / MAX  + 300 );
	    size_ = ta;
	    size = ta; }
	}
	


	return(Offtime_.value());
}



void EXPOO_IP_Traffic::timeout()
{
  if (! running_)
    return;

  /* sends a packet */  
  agent_->sendmsg(size_);

  // figure out when to send the next one 
  nextPkttime_ = next_interval(size_);
  // schedule it 
  if (nextPkttime_ > 0)
    timer_.resched(nextPkttime_);
}

